#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>

#define WIFI_CONNECT_DEBUG_PIN D0
#define SERIAL_MESSAGE_DEBUG_PIN D1
#define IFTTT_CONNECT_DEBUG_PIN D2

#define IFTT_SERVICE_TIMEOUT_ATTEMPTS 3
#define IFTT_SERVICE_DELAY 3000
#define IFTT_SERVICE_RETRY_DELAY 1000

#define WIFI_CONNECT_DELAY 1000
#define DELAY 1000

#define HTTPS_PORT 443

int iftt_connect_attempts = 0;
int wifi_connect_attempts = 0;

const char *ssid = "";        // change
const char *password = "";    // change

const char* server = "maker.ifttt.com";
const char fingerprint[] PROGMEM = "b871167c629f5eaa3346832a7e95ad4be6b2b807";
const char* resource = "https://maker.ifttt.com/trigger/plant_notify/with/key/mFPcx-7gCyM0sj0S0lTnx6FMxgz7KcX10ErmLFyN1hu";   // change with your resource

const char* sad_plant_photo = "https://drive.google.com/file/d/1pnIbCDhW1TPCMDkwLS5GEEeToJBJjhPq/view?usp=sharing";   // change with a photo of your choice

const char* LIGHT = "light";
const char* TEMP = "temp";
const char *HUM = "hum";
const char *SOIL_HUM = "soil";

String message;
char msg[100];
char* token = ",";

WiFiClientSecure client;

void setup_debug_leds() {
    pinMode(IFTTT_CONNECT_DEBUG_PIN, OUTPUT);
    pinMode(SERIAL_MESSAGE_DEBUG_PIN, OUTPUT);
    pinMode(WIFI_CONNECT_DEBUG_PIN, OUTPUT);
    
    digitalWrite(WIFI_CONNECT_DEBUG_PIN, LOW);
    digitalWrite(SERIAL_MESSAGE_DEBUG_PIN, LOW);
    digitalWrite(IFTTT_CONNECT_DEBUG_PIN, LOW);
}

void setup_connections() {
    WiFi.mode(WIFI_STA);                                     // set the WiFi mode to WiFi_STA (the board will act as a station)
    client.setFingerprint(fingerprint);                      // IFTTT certificate fingerprint
    connectToWiFi();
}

void setup() {
    Serial.begin(9600);
    
    setup_debug_leds();
    setup_connections();  
}

void connectToWiFi(){ 
    
    digitalWrite(WIFI_CONNECT_DEBUG_PIN, LOW);               // WiFi not connected
               
    WiFi.begin(ssid, password);                              // connect to the WiFi network using the ssid and password strings
    
    while (WiFi.status() != WL_CONNECTED) {
        delay(WIFI_CONNECT_DELAY);
        Serial.print(".");
    }
    
    digitalWrite(WIFI_CONNECT_DEBUG_PIN, HIGH);              // WiFi connection was succesfull
}

void loop() {

    if(Serial.available()) {                                 // message arrived from Arduino Uno
        digitalWrite(SERIAL_MESSAGE_DEBUG_PIN, HIGH);         
    
        if ((WiFi.status() != WL_CONNECTED)) {      
            WiFi.disconnect();
            connectToWiFi();
        }
    
        message = Serial.readStringUntil('\n');
        message.toCharArray(msg, message.length());
        msg[message.length()] = '\0';
    
        char* substr = NULL;
        substr = strtok(msg,token);

        int chunk_length = 0;
        String send_message = "value1=";
        send_message += "Hei!_Am_nevoie_de_";
        int count = 0;
    
        while((substr != NULL)) {                             // build message for the web request
            Serial.println(substr);
            if (strncmp(substr, LIGHT, 5) == 0) {
              send_message += "lumina";
              count++;
            } else if (strncmp(substr, TEMP, 4) == 0) {
              if(count) send_message +="_si_de_";
              send_message += "caldura";
              count++;
            } else if (strncmp(substr, HUM, 3) == 0) {
              if(count) send_message +="_si_de_";
              send_message += "umiditate";
              count++;
            } else if (strncmp(substr, SOIL_HUM, 4) == 0) {
              if(count) send_message +="_si_de_";
              send_message += "apa";
              count++;
            }
            substr = strtok(NULL,token);
        }

        send_message += "!";
        send_message += "&value2=";
        send_message += sad_plant_photo;

        chunk_length = send_message.length();

        Serial.println(send_message);
        
        while(1) {
          
            iftt_connect_attempts++;                                                      // try to connect to ifttt server
              
            if (client.connect(server, HTTPS_PORT)) {                                     // connection to ifttt server succesfull
              
                iftt_connect_attempts = 0;                                               
                
                digitalWrite(IFTTT_CONNECT_DEBUG_PIN, HIGH);                              

                client.print(String("POST ") + resource +  " HTTP/1.1\r\n" + "Host: "     // send web request
                            + server + "\r\n" + "Connection: close\r\n" +
                            "Content-Type: application/x-www-form-urlencoded\r\n"
                            + "Content-Length: " + chunk_length + "\r\n\r\n"
                            + send_message  + "\r\n");
                
                delay(IFTT_SERVICE_DELAY);
          
                while(client.available()){                                                // read all the lines of the reply from server and print them to Serial - for debugging
                  String line = client.readStringUntil('\r');
                  Serial.print("line:");
                  Serial.print(line);
                }
                
                break;
          
            } else {
                if(iftt_connect_attempts == IFTT_SERVICE_TIMEOUT_ATTEMPTS) {              // connection to ifttt server was unsuccesful
                    digitalWrite(IFTTT_CONNECT_DEBUG_PIN, LOW);
                    iftt_connect_attempts = 0;
                    break;
                }
                delay(IFTT_SERVICE_RETRY_DELAY);
            }
        }
    
        digitalWrite(IFTTT_CONNECT_DEBUG_PIN, LOW);
        digitalWrite(SERIAL_MESSAGE_DEBUG_PIN, LOW);
    }

    delay(DELAY);
}
